@@ -38,6 +38,7 @@ default_key_word_success = ['success', 'ok', 'done', 'yes', 'okay', 'true', 'com
 default_key_word_fails = ['fail', 'false', 'error', 'bug', 'fire']
 default_color_list = [Fore.YELLOW, Fore.BLUE, Fore.MAGENTA, Fore.CYAN, Fore.LIGHTYELLOW_EX, Fore.LIGHTBLUE_EX,
                       Fore.LIGHTMAGENTA_EX, Fore.LIGHTCYAN_EX]
+
 last_logger: [Logger | None] = None
 
 
@@ -53,53 +54,54 @@ BOLD = '\033[1m'
 class Logger:
 
     def __init__(self, log_name: [str | None] = 'logger', log_level: LogLevel = LogLevel.INFO,
-                 skip_repeat: bool = False, handler: [Callable | List[Callable]] = None, stage_sep: str = '..',
-                 timestamp: [str | None] = "%m.%d %H:%M:%S", key_word_success: [list | None] = None,
-                 key_word_fails: [list | None] = None, stage_color_list: [List[Fore] | None] = None):
+                 skip_repeat: bool = False, stage_sep: str = '..', timestamp: [str | None] = "%m.%d %H:%M:%S",
+                 callback: [Callable | List[Callable]] = None, stage_color_list: [List[Fore] | None] = None,
+                 key_word_success: [list | None] = None, key_word_fails: [list | None] = None):
+
         """
         :param log_name: the display name of current logger.
         :param log_level: (Optional) (Default: Logger.INFO)the log level of current logger.
-        :param handler: (Optional) the handlers of current logger. you can get the output msg from the handlers.
+        :param callback: (Optional) the _callback_list of current logger. you can get the output msg from the _callback_list.
         :param skip_repeat: (Optional) if True, the current logger will skip the repeat msg.
-        :param timestamp: (Optional) the timestamp format of current logger.
+        :param timestamp: (Optional) the _timestamp format of current logger.
         :param stage_sep: (Optional) the separator of stage.
         :param key_word_success: (Optional) the key words of success.
         :param key_word_fails: (Optional) the key words of fails.
         :param stage_color_list: (Optional) the color list of stage.
         """
 
-        self.log_name = log_name
-        if not self.log_name:
-            self.log_name = ''
+        self._log_name = log_name
+        if not self._log_name:
+            self._log_name = ''
         else:
-            self.log_name = f'[{self.log_name}]'
+            self._log_name = f'[{self._log_name}]'
 
         if not isinstance(log_level, LogLevel):
             raise TypeError(f'Error log level type: {type(log_level)}')
 
-        self.log_level = log_level
+        self._log_level = log_level
 
-        self.handlers = []
-        if handler is not None:
-            if not isinstance(handler, list):
-                handler = [handler]
-            for h in handler:
+        self._callback_list = []
+        if callback is not None:
+            if not isinstance(callback, list):
+                callback = [callback]
+            for h in callback:
                 if not callable(h):
-                    raise TypeError('Handler must be callable!!')
-            self.handlers = handler
+                    raise TypeError('callback must be callable!!')
+            self._callback_list = callback
 
-        self.skip_repeat = skip_repeat
-        self.timestamp = timestamp
+        self._skip_repeat = skip_repeat
+        self._timestamp = timestamp
 
-        self.stage_sep = stage_sep
+        self._stage_sep = stage_sep
 
         if key_word_success is None:
             key_word_success = default_key_word_success
-        self.key_word_success = key_word_success
+        self._key_word_success = key_word_success
 
         if key_word_fails is None:
             key_word_fails = default_key_word_fails
-        self.key_word_fails = key_word_fails
+        self._key_word_fails = key_word_fails
 
         self._stage_count = 0
 
@@ -108,53 +110,52 @@ class Logger:
         else:
             self._stage_color_list = stage_color_list
 
-        self.status = LoggerStatus.FINISH
+        self._logger_status = LoggerStatus.FINISH
         self._last_msg = None
-        self._stage_level = None
+        self._stage_loglevel = None
 
     def info(self, *msg):
-        self._start_check(LogLevel.INFO, *msg)
+        self._start(LogLevel.INFO, *msg)
 
     def debug(self, *msg):
-        self._start_check(LogLevel.DEBUG, *msg)
+        self._start(LogLevel.DEBUG, *msg)
 
     def trace(self, *msg):
-        self._start_check(LogLevel.TRACE, *msg)
-
-    def _start_check(self, log_level: LogLevel, *msg):
-
-        with global_lock:
-            if self._start(log_level, *msg):
-                self.status = LoggerStatus.STAGE
-                self._stage_level = log_level
+        self._start(LogLevel.TRACE, *msg)
 
     def stage(self, *msg):
         # its log level is the same as the last do_level
-        if self.status == LoggerStatus.FINISH:
+
+        if last_logger and last_logger._logger_status == LoggerStatus.FINISH:
             # works like normal logger
-            self._start_check(LogLevel.INFO, *msg)
+            self._start(self._stage_loglevel if self._stage_loglevel else LogLevel.INFO, *msg)
+            return
+        if self._logger_status == LoggerStatus.FINISH:
+            self._start(self._stage_loglevel if self._stage_loglevel else LogLevel.INFO, *msg)
             return
 
-        self._stage(self._stage_level, *msg)
+        self._stage(self._stage_loglevel, *msg)
 
     def _print(self, *args, **kwargs):
         with global_lock:
-            self.status = LoggerStatus.PRINT
+            self._logger_status = LoggerStatus.PRINT
             self._add_newline()
             self._output(None, *args, **kwargs)
-            self.status = LoggerStatus.FINISH
+            self._logger_status = LoggerStatus.FINISH
 
             global last_logger
             last_logger = self
 
-
     def _check_log_level(self, log_level: LogLevel) -> bool:
         # check the msg will be output or not
 
-        if self.status == LoggerStatus.PRINT:
+        if self._logger_status == LoggerStatus.PRINT:
             return True
 
-        if self.log_level > log_level:
+        if not log_level:
+            raise ValueError('log_level must be set!!')
+
+        if self._log_level > log_level:
             return False
 
         return True
@@ -163,21 +164,21 @@ class Logger:
         if not self._check_log_level(log_level):
             return
 
-        if self.status not in [LoggerStatus.STAGE, LoggerStatus.START]:
-            raise Exception(f'Unknown log status {self.status}')
+        if self._logger_status not in [LoggerStatus.STAGE, LoggerStatus.START]:
+            raise Exception(f'Unknown log _logger_status {self._logger_status}')
 
         message = str(msg)
-        if self.skip_repeat:
+        if self._skip_repeat:
             self._last_msg = message
 
         color = ''
-        for s in self.key_word_success:
+        for s in self._key_word_success:
             if s in message.lower():
                 color = Fore.GREEN
                 break
 
         if not color:
-            for s in self.key_word_fails:
+            for s in self._key_word_fails:
                 if s in message.lower():
                     color = Fore.RED
                     break
@@ -186,73 +187,78 @@ class Logger:
             color = self._stage_color_list[self._stage_count]
             self._stage_count = (self._stage_count + 1) % len(self._stage_color_list)
 
-        total_message = f' {self.stage_sep} {color}{BOLD}{message}'
+        total_message = f' {self._stage_sep} {color}{BOLD}{message}'
 
         utils.output_screen(total_message)
-        utils.output_file(self.handlers, total_message)
+        utils.callback(self._callback_list, total_message)
 
         global last_logger
         last_logger = self
 
     def _start(self, log_level: LogLevel, *msg) -> bool:
 
-        if not self._check_log_level(log_level):
-            return False
-
-        self.status = LoggerStatus.START
-
-        if not msg:
-            msg = ' '
-
-        message = f'{merge_msg(msg[0], frame=False)}'
-        for m in msg[1:]:
-            message = f'{message} {merge_msg(m)}'
-
-        if self.skip_repeat:
-            if self._last_msg == message:
+        with global_lock:
+            if not self._check_log_level(log_level):
                 return False
-            self._last_msg = message
 
-        location = ''
-        if self.log_level <= LogLevel.DEBUG:
-            cf = inspect.currentframe()
-            line_no = cf.f_back.f_back.f_lineno
-            file_name = cf.f_back.f_back.f_code.co_filename
-            file_name = os.path.basename(file_name)
-            location = f'[{file_name} {line_no}]'
+            self._logger_status = LoggerStatus.START
 
-        timestamp = f'[{strftime(self.timestamp)}]' if self.timestamp else ''
-        total_message = f'{timestamp}{self.log_name}{location} {message}'.strip()
+            if not msg:
+                msg = ' '
 
-        self._add_newline()
-        self._output(total_message)
+            message = f'{merge_msg(msg[0], frame=False)}'
+            for m in msg[1:]:
+                message = f'{message} {merge_msg(m)}'
 
-        global last_logger
-        last_logger = self
+            if self._skip_repeat:
+                if self._last_msg == message:
+                    return False
+                self._last_msg = message
 
-        return True
+            location = ''
+            if self._log_level <= LogLevel.DEBUG:
+                cf = inspect.currentframe()
+                line_no = cf.f_back.f_back.f_lineno
+                file_name = cf.f_back.f_back.f_code.co_filename
+                file_name = os.path.basename(file_name)
+                location = f'[{file_name} {line_no}]'
+
+            timestamp = f'[{strftime(self._timestamp)}]' if self._timestamp else ''
+            total_message = f'{timestamp}{self._log_name}{location} {message}'.strip()
+
+            self._add_newline()
+            self._output(total_message)
+
+            global last_logger
+            last_logger = self
+
+            self._logger_status = LoggerStatus.STAGE
+            self._stage_loglevel = log_level
+
+            return True
 
     def __add_newline(self) -> None:
         old_print()
         if last_logger is not self:
-            last_logger.status = LoggerStatus.FINISH
+            last_logger._logger_status = LoggerStatus.FINISH
+            last_logger._stage_loglevel = None
         self._stage_count = 0
 
     def _add_newline(self):
 
-        if self.status == LoggerStatus.STAGE:
+        if self._logger_status == LoggerStatus.STAGE:
             # if the last stage is stage, don't need to lock
-            raise Exception('Cannot print in stage status')
+            raise Exception('Cannot print in stage _logger_status')
 
         global last_logger
         if last_logger:
-            if self.status == LoggerStatus.START:
-                # if the status is start, it means we need to check the status of last logger
+            if self._logger_status == LoggerStatus.START:
+                # if the _logger_status is start, it means we need to check the _logger_status of last logger
                 # note: last logger could be self
-                if last_logger.status != LoggerStatus.FINISH:
+                if last_logger._logger_status != LoggerStatus.FINISH:
                     # if self or last logger is not finish, add newline
                     self.__add_newline()
-            elif self.status == LoggerStatus.PRINT:
+            elif self._logger_status == LoggerStatus.PRINT:
                 # is the last print is not print, we need to add newline
                 # adjust this to avoid the situation that the last print is print
                 if last_logger.__class__.__name__ != 'PrintLogger':
@@ -260,13 +266,12 @@ class Logger:
                     self.__add_newline()
 
     def _output(self, total_message: [str | None], *args, **kwargs) -> None:
-        if self.status == LoggerStatus.PRINT:
+        if self._logger_status == LoggerStatus.PRINT:
             old_print(*args, **kwargs)
             return
 
         utils.output_screen(total_message)
-        utils.output_file(self.handlers, total_message)
-
+        utils.callback(self._callback_list, total_message)
 
 
 class PrintLogger(Logger):
@@ -274,4 +279,4 @@ class PrintLogger(Logger):
         self._print(*args, **kwargs)
 
 
-print_logger = PrintLogger(log_name='', timestamp=None)
+print_logger = PrintLogger(log_name='', timestamp='')
